#pragma once
#include "atlmisc.h"
#include <map>

class CCritSec
{

private:

    CCritSec(const CCritSec &refCritSec);
    CCritSec &operator=(const CCritSec &refCritSec);

    CRITICAL_SECTION m_CritSec;

public:
    CCritSec() 
    {
        InitializeCriticalSection(&m_CritSec);
    };

    ~CCritSec() 
    {
        DeleteCriticalSection(&m_CritSec);
    }

    void Lock()
    {
        EnterCriticalSection(&m_CritSec);
    };

    void Unlock()
    {
        LeaveCriticalSection(&m_CritSec);
    };
};

// locks a critical section, and unlocks it automatically
// when the lock goes out of scope
class CAutoLock
{
private:

    CAutoLock(const CAutoLock &refAutoLock);
    CAutoLock &operator=(const CAutoLock &refAutoLock);

protected:
    CCritSec * m_pLock;

public:
    CAutoLock(CCritSec * plock)
    {
        m_pLock = plock;
        m_pLock->Lock();
    };

    ~CAutoLock()
    {
        m_pLock->Unlock();
    };
};

// Used to map file contents into memory
class CFileMemoryMapping
{

public:
    CFileMemoryMapping();
    ~CFileMemoryMapping();

    BOOL Init(LPCTSTR szFileName);
    BOOL Destroy();

    ULONG64 GetSize();
    LPBYTE CreateView(DWORD dwOffset, DWORD dwLength);

private:

    HANDLE m_hFile;             // Handle to current file
    HANDLE m_hFileMapping;      // Memory mapped object
    DWORD m_dwAllocGranularity; // System allocation granularity
    ULONG64 m_uFileLength;      // Size of the file
    CCritSec m_csLock;          // Sync object

    std::map<DWORD, LPBYTE> m_aViewStartPtrs; // Base of the view of the file
};

class CFilePreviewCtrl : public CWindowImpl<CFilePreviewCtrl, CStatic>
{
public:

    CFilePreviewCtrl();
    ~CFilePreviewCtrl();

    // Sets current file and preview mode. You can pass NULL as file name to clear the preview.
    BOOL SetFile(LPCTSTR szFileName);

    // Sets the text to display when nothing to preview (the default is "No data to display")
    void SetEmptyMessage(CString sText);

    // Sets count of bytes per line for Hex preview
    BOOL SetBytesPerLine(int nBytesPerLine);

protected:

    DECLARE_WND_SUPERCLASS(NULL, CWindow::GetWndClassName())

    BEGIN_MSG_MAP(CFilePreviewCtrl)
        if (uMsg == WM_NCHITTEST ||
            uMsg == WM_NCLBUTTONDOWN ||
            uMsg == WM_NCMBUTTONDOWN ||
            uMsg == WM_NCXBUTTONDOWN ||
            uMsg == WM_NCLBUTTONDBLCLK)
        {
            // This is to enable scroll bar messages
            bHandled = TRUE;
            lResult = ::DefWindowProc(m_hWnd, uMsg, wParam, lParam);
            return TRUE;
        }

        MESSAGE_HANDLER(WM_DESTROY, OnDestroy)
        MESSAGE_HANDLER(WM_ERASEBKGND, OnEraseBkgnd)
        MESSAGE_HANDLER(WM_PAINT, OnPaint)
        MESSAGE_HANDLER(WM_SIZE, OnSize)
        MESSAGE_HANDLER(WM_HSCROLL, OnHScroll)
        MESSAGE_HANDLER(WM_VSCROLL, OnVScroll)
        MESSAGE_HANDLER(WM_RBUTTONUP, OnRButtonUp)
    END_MSG_MAP()

    LRESULT OnDestroy(UINT /*uMsg*/, WPARAM /*wParam*/, LPARAM /*lParam*/, BOOL& /*bHandled*/);
    LRESULT OnEraseBkgnd(UINT /*uMsg*/, WPARAM /*wParam*/, LPARAM /*lParam*/, BOOL& /*bHandled*/);
    LRESULT OnPaint(UINT /*uMsg*/, WPARAM wParam, LPARAM /*lParam*/, BOOL& /*bHandled*/);
    LRESULT OnSize(UINT /*uMsg*/, WPARAM wParam, LPARAM /*lParam*/, BOOL& /*bHandled*/);
    LRESULT OnHScroll(UINT /*uMsg*/, WPARAM wParam, LPARAM /*lParam*/, BOOL& /*bHandled*/);
    LRESULT OnVScroll(UINT /*uMsg*/, WPARAM wParam, LPARAM /*lParam*/, BOOL& /*bHandled*/);
    LRESULT OnRButtonUp(UINT /*uMsg*/, WPARAM wParam, LPARAM /*lParam*/, BOOL& /*bHandled*/);

    void SetupScrollbars();
    CString FormatHexLine(LPBYTE pData, int nBytesInLine, ULONG64 uLineOffset);
    void DrawHexLine(HDC hdc, DWORD nLineNo);
    void DoPaintEmpty(HDC hDC);
    void DoPaintHex(HDC hDC);

    CCritSec m_csLock;        // Critical section
    CFileMemoryMapping m_fm;  // File mapping object
    HFONT m_hFont;            // Font in use
    int m_xChar;              // Size of character in x direction
    int m_yChar;              // Size of character in y direction
    int m_nMaxColsPerPage;    // Maximum columns per page
    int m_nMaxLinesPerPage;   // Maximum count of lines per one page
    int m_nMaxDisplayWidth;
    ULONG64 m_uNumLines;      // Number of lines in the file
    int m_nBytesPerLine;      // Count of displayed bytes per line
    int m_cchTabLength;
    CString m_sEmptyMsg;      // Text to display when file is empty
    int m_nHScrollPos;        // Horizontal scroll position
    int m_nHScrollMax;        // Max horizontal scroll position
    int m_nVScrollPos;        // Vertical scrolling position
    int m_nVScrollMax;        // Maximum vertical scrolling position
};
